/* This file is part of Ganv.
 * Copyright 2007-2016 David Robillard <http://drobilla.net>
 *
 * Ganv is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * Ganv is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Ganv.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef GANV_TYPES_H
#define GANV_TYPES_H

typedef struct _GanvCanvas GanvCanvas;
typedef struct _GanvEdge   GanvEdge;
typedef struct _GanvModule GanvModule;
typedef struct _GanvNode   GanvNode;
typedef struct _GanvPort   GanvPort;
typedef struct _GanvBox    GanvBox;

#endif  /* GANV_TYPES_H */
